% 07/17/2016
%% This procedure computes the likelihood function associated with a nonsingular submodel 
% Adpated from Lubik and Schorfheide's (2004) gauss code sunlh.src; 6/12/3003

%% -----Input-----
%TT,TC,TEPS,TETA,RC: output from solving the model
%para: the parameter associated with the full model; those absent from the
%submodel are already replaced by zeros prior to calling this procedure.
%Ydata: observations
%sel: indictors to select the variables in YY for constructing the likelihood
%M: a diagonal matrix to select shocks. If there is one shock (say the
%first shock), then only the first element of M is nonzero.

%% ---Output
%lvec: a T by 1 vector containing log likelihood components
%loglh: log likelihood value

function [lvec,loglh,retcode,obsmean,obsvar]=evalcgg_s(TT,TC,TEPS,TETA,RC,para,Ydata,sel,M)

% retcode = -1 : non existence or non-uniqueness             
%         = 0  : existence and uniqueness            
% Parameters: [1/tau rstar kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz, rhogz, pistar]';


npara  = rows(para);

nobs      = rows(Ydata);
loglh     = 0;
loglhzero = -1E10;

if isempty(TT)
    disp('empty solution: nonexistence/numerical issue') 
    loglh = loglhzero;
    retcode=-1;
    return
end

nstate = cols(TT);

if RC==[1;1]
       
  % /* determinacy */
    retcode = 0;
    RR = TEPS;
    sigr=para(9);
    sigg   = para(10);
    siga   = para(11);
    r_g_a   = para(12);
    omega = zeros(3,3);
    omega(1,1) = sigr^2;
    omega(2,2) = sigg^2;
    omega(3,3)=siga^2;
    omega(2,3) = r_g_a*sigg*siga;
    omega(3,2) = omega(2,3);
    QQ=omega;

else
   %/* no equilibrium exists, or indeterminacy, the code terminates */
   display ('error; indeterminacy or no solution');
   loglh = loglhzero;
   retcode=-1;
   return
end
    

%/* create system matrices for state space model
%*/

%/* composition of state vector
%**   1.  y  : output
%**   2.  inf: pi, inflation
%**   3.  r2 : R, nominal interest rate
%*/
%Steady state: DD = [0; para(13); para(2)+para(13)];

ZZ0      = zeros(3,nstate);
ZZ0(1,1) = 1; 
ZZ0(2,2) = 4; %/* measured inflation is annualized */ 
ZZ0(3,3) = 4; %/* measured interest rates are annualized */
DD0 = [0; para(13); para(2)+para(13)];

%specify the submodel
ZZ=sel*ZZ0; %select the steady states
DD=sel*DD0; 

QQ=M*QQ*M'; %zero out the irrelevent shocks

YY=Ydata*sel'; %select observables  

obsmean   = zeros(nobs,cols(YY));
obsvar    = zeros(nobs,cols(YY));

HH = zeros(cols(YY),cols(YY));
%VV = zeros(cols(YY),cols(YY));
VV = zeros(cols(RR),cols(QQ));

% Check whether covariance matrix QQ is positive semi definite

if sum(eig(QQ)<-1E-10) > 0;
   loglh = loglhzero;
   return
end

% Define the initial mean and variance for the state vector
At = zeros(nstate,1);
Pt = dlyap2(TT,RR*QQ*RR');
 
lvec=zeros(nobs,1);
for t=1:nobs
   At1 = At;
   Pt1 = Pt;

   % Forecasting
   
   alphahat = TT*At1;
   Phat = TT*Pt1*TT' + RR*QQ*RR';
   
   yhat = ZZ*alphahat + DD;
   nu   = YY(t,:) - yhat';
   
   Ft   = ZZ*Phat*ZZ' + HH + ZZ*RR*VV*sel' + (ZZ*RR*VV*sel')';
   Ft   = 0.5*(Ft + Ft');
   
   loglh = loglh -0.5*cols(YY)*log(2*pi) - 0.5*log(abs(det(Ft))) - 0.5*nu*inv(Ft)*nu';
   lvec(t)=-0.5*cols(YY)*log(2*pi) - 0.5*log(abs(det(Ft))) - 0.5*nu*inv(Ft)*nu';

  % Updating
   At = alphahat + (Phat*ZZ' + RR*VV*sel')*inv(Ft)*nu';
   Pt = Phat - (Phat*ZZ'+RR*VV*sel')*inv(Ft)*(Phat*ZZ'+RR*VV*sel')';

   obsmean(t,:) = yhat';
   obsvar(t,:)  = (diag(Ft))';
end 
   
loglh=real(loglh);

end